import { useRef, useState, useEffect } from 'react';
import CryptoJs from 'crypto-js';
import encHex from 'crypto-js/enc-hex';

/**
 * hash file
 * @param file the file need to hash
 * @param hashAlgo hash algorithm 'md5' |'sha1'| 'sha256'  default:sha256
 * @param chunkSizeCustom   chunk size default:10M
 * @return IUseHashFileReturn
 */

var useFileHashCode = function useFileHashCode(file, hashAlgo, chunkSizeCustom) {
  var startTimeRef = useRef(null);

  var _useState = useState(true),
      isHashLoading = _useState[0],
      setIsHashLoading = _useState[1];

  var _useState2 = useState(false),
      isHashError = _useState2[0],
      setIsHashError = _useState2[1];

  var _useState3 = useState(null),
      md5 = _useState3[0],
      setMd5 = _useState3[1];

  var _useState4 = useState(null),
      sha1 = _useState4[0],
      setSha1 = _useState4[1];

  var _useState5 = useState(null),
      sha256 = _useState5[0],
      setSha256 = _useState5[1];

  var _useState6 = useState(null),
      sha512 = _useState6[0],
      setSha512 = _useState6[1];

  var _useState7 = useState(null),
      timeSpend = _useState7[0],
      setTimeSpend = _useState7[1];

  useEffect(function () {
    /* file is null init all state */
    if (!file) {
      setIsHashLoading(true);
      setIsHashError(false);
      setMd5(null);
      setSha1(null);
      setSha256(null);
      setSha512(null);
      setTimeSpend(null);
      return;
    }
    /* record start time */


    startTimeRef.current = Date.now();

    var hashFileInternal = function hashFileInternal(file, alog) {
      var chunkSize = 1024 * 1024 * 10;
      /* custom chunk size */

      if (chunkSizeCustom) {
        chunkSize = chunkSizeCustom;
      }

      var promise = Promise.resolve();

      var _loop = function _loop(index) {
        promise = promise.then(function () {
          return hashBlob(file.slice(index, index + chunkSize));
        });
      };

      for (var index = 0; index < file.size; index += chunkSize) {
        _loop(index);
      }
      /* arraybuffer to word array */


      function arrayBufferToWordArray(ab) {
        var i8a = new Uint8Array(ab);
        var a = [];

        for (var i = 0; i < i8a.length; i += 4) {
          a.push(i8a[i] << 24 | i8a[i + 1] << 16 | i8a[i + 2] << 8 | i8a[i + 3]);
        }

        return CryptoJs.lib.WordArray.create(a, i8a.length);
      }
      /* hash blob */


      var hashBlob = function hashBlob(blob) {
        return new Promise(function (resolve) {
          var reader = new FileReader();

          reader.onload = function (_ref) {
            var target = _ref.target;
            if (!target.result) return;
            if (typeof target.result === 'string') return;
            var wordArray = arrayBufferToWordArray(target.result); // update hash

            alog.update(wordArray);
            resolve();
          };

          reader.readAsArrayBuffer(blob);
        });
      };

      return promise.then(function () {
        return encHex.stringify(alog.finalize());
      });
    };

    var hashFileSha256 = function hashFileSha256() {
      hashFileInternal(file, CryptoJs.algo.SHA256.create()).then(function (sha256) {
        setSha256(sha256);

        if (startTimeRef.current) {
          var t = Date.now() - startTimeRef.current;
          setTimeSpend(t);
        }
      })["catch"](function () {
        setIsHashError(true);
        setIsHashLoading(false);
      });
    };
    /* chose algo */


    switch (hashAlgo) {
      case 'md5':
        hashFileInternal(file, CryptoJs.algo.MD5.create()).then(function (md5) {
          setMd5(md5);

          if (startTimeRef.current) {
            var t = Date.now() - startTimeRef.current;
            setTimeSpend(t);
          }
        })["catch"](function () {
          setIsHashError(true);
          setIsHashLoading(false);
        });
        break;

      case 'sha1':
        hashFileInternal(file, CryptoJs.algo.SHA1.create()).then(function (sha1) {
          setSha1(sha1);

          if (startTimeRef.current) {
            var t = Date.now() - startTimeRef.current;
            setTimeSpend(t);
          }
        })["catch"](function () {
          setIsHashError(true);
          setIsHashLoading(false);
        });
        break;

      case 'sha256':
        hashFileSha256();
        break;

      case 'sha512':
        hashFileInternal(file, CryptoJs.algo.SHA512.create()).then(function (sha512) {
          setSha512(sha512);

          if (startTimeRef.current) {
            var t = Date.now() - startTimeRef.current;
            setTimeSpend(t);
          }
        })["catch"](function () {
          setIsHashError(true);
          setIsHashLoading(false);
        });
        break;

      default:
        hashFileSha256();
    }
  }, [chunkSizeCustom, file, hashAlgo]);
  return {
    isHashLoading: isHashLoading,
    isHashError: isHashError,
    md5: md5,
    sha1: sha1,
    sha256: sha256,
    sha512: sha512,
    timeSpend: timeSpend
  };
};
var useStringHashCode = function useStringHashCode(msg, hashAlgo) {
  switch (hashAlgo) {
    case 'md5':
      return CryptoJs.MD5(msg).toString();

    case 'sha1':
      return CryptoJs.SHA1(msg).toString();

    case 'sha256':
      return CryptoJs.SHA256(msg).toString();

    case 'sha512':
      return CryptoJs.SHA512(msg).toString();

    default:
      return CryptoJs.SHA256(msg).toString();
  }
};

export { useFileHashCode, useStringHashCode };
//# sourceMappingURL=use-hashcode.esm.js.map
